from spark_advisor_models.config import Thresholds
from spark_advisor_models.model import JobAnalysis, RuleResult
from spark_advisor_rules.rules import Rule, rules_for_threshold


class StaticAnalysisService:
    def __init__(self, rules: list[Rule] | None = None, *, thresholds: Thresholds | None = None) -> None:
        if rules is not None and thresholds is not None:
            raise ValueError("Provide either 'rules' or 'thresholds', not both")
        if rules is not None:
            self._rules = rules
        else:
            from spark_advisor_models.defaults import DEFAULT_THRESHOLDS

            self._rules = rules_for_threshold(thresholds or DEFAULT_THRESHOLDS)

    def analyze(self, job: JobAnalysis) -> list[RuleResult]:
        all_results: list[RuleResult] = []
        for rule in self._rules:
            all_results.extend(rule.evaluate(job))

        all_results.sort(key=lambda r: r.severity.order)

        return all_results



import pytest

from spark_advisor_models.model import JobAnalysis, StageMetrics
from spark_advisor_models.testing import make_executors, make_job, make_stage


@pytest.fixture
def default_job() -> JobAnalysis:
    return make_job()


@pytest.fixture
def job_with_executors() -> JobAnalysis:
    return make_job(executors=make_executors())


@pytest.fixture
def skewed_stage() -> StageMetrics:
    return make_stage(stage_id=0, run_time_median=10, run_time_max=500)


@pytest.fixture
def spilling_stage() -> StageMetrics:
    return make_stage(stage_id=0, spill_to_disk_bytes=2 * 1024**3)


@pytest.fixture
def high_gc_stage() -> StageMetrics:
    return make_stage(stage_id=0, task_count=100, sum_executor_run_time_ms=100_000, total_gc_time_ms=50_000)
